/*
 * SampleBasicPDFTimeStampGenerator.java
 *
 * Copyright(C) 2021 AMANO Corporation. All Rights Reserved.
 *
 * PDF Lib-J API - TvvO
 *
 * w肳ꂽ PDF t@Cɑ΂ PDF ^CX^v𖄂ߍރR\[vOłB
 *
 * ZLeB̏̕ꍇ
 * Usage: java SampleBasicPDFTimeStampGenerator PDFFilePath Server UserName Password
 *
 *    PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX
 *    Server            - ^CX^vT[o
 *    UserName          - BasicF؂̃[U[
 *    Password          - BasicF؂̃pX[h
 *
 * PDF WZLeBL̏̕ꍇ
 * Usage: java SampleBasicPDFTimeStampGenerator PDFFilePath UserPassword OwnerPassword Server UserName Password
 *
 *    PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX
 *    UserPassword      - [U[pX[h
 *    OwnerPassword     - I[i[pX[h
 *    Server            - ^CX^vT[o
 *    UserName          - BasicF؂̃[U[
 *    Password          - BasicF؂̃pX[h
 */

import java.io.*;

import jp.co.amano.etiming.apl3161.*;
import jp.co.amano.etiming.atss3161.UsernamePasswordCredentials;

public class SampleBasicPDFTimeStampGenerator {

	public static void main(String[] args) throws Exception {

		String pdfFilePath = null;
		char[] ownerPassword = null;
		char[] userPassword = null;
		String timeStampServer = null;
		String userName = null;
		String password = null;

		// `FbN
		if (args.length == 4) {
			pdfFilePath = args[0];
			timeStampServer = args[1];
			userName = args[2];
			password = args[3];
		} else if (args.length == 6) {
			pdfFilePath = args[0];
			userPassword = args[1].toCharArray();
			ownerPassword = args[2].toCharArray();
			timeStampServer = args[3];
			userName = args[4];
			password = args[5];
		} else {
			System.out.println("usage : java SampleBasicPDFTimeStampGenerator <PDFFilePath> <Server> <UserName> <Password>");
			System.out.println("      : java SampleBasicPDFTimeStampGenerator <PDFFilePath> <UserPassword> <OwnerPassword> <Server> <UserName> <Password>");
			System.out.println();
			System.out.println("      PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX");
			System.out.println("      UserPassword      - PDF ̃[U[pX[h");
			System.out.println("      OwnerPassword     - PDF ̃I[i[pX[h");
			System.out.println("      Server            - ^CX^vT[o");
			System.out.println("      UserName          - BasicF؂̃[U[");
			System.out.println("      Password          - BasicF؂̃pX[h");
			System.exit(1);
		}

		try {
			// ^CX^vg[N擾ۂɎgp^CX^vvgR
			// IuWFNg𐶐
			HTTPTimeStampProtocol protocol = new HTTPTimeStampProtocol(timeStampServer);
			
			// BasicF؃[U[ݒ肷
			UsernamePasswordCredentials credentials = new UsernamePasswordCredentials(userName, password);
			protocol.setBasicCredentials(credentials);

			// PDF ^CX^vIuWFNg𐶐
			PDFTimeStampGenerator generator = new PDFTimeStampGenerator(protocol);

			// ^C][ݒ
			generator.setTimeZone(DefinedTimeZone.JST);

			// ^CX^vg[N؃IuWFNg̐
			NoTSACertValidationTSTValidator validator = new NoTSACertValidationTSTValidator();
			generator.setTSTValidator(validator);

			// eȂ
			generator.setImprintPasting(null);

			// PDF t@C̃I[v
			RandomAccessFile pdfDocument = new RandomAccessFile(pdfFilePath, "r");

			// PDF ^CX^v𐶐
			InputStream stamped = null;
			if (ownerPassword == null && userPassword == null) {
				stamped = generator.generate(pdfDocument);
			} else {
				stamped = generator.generate(pdfDocument, userPassword, ownerPassword);
			}
			pdfDocument.close();

			// PDF ^CX^v𖄂ߍ PDF t@C̕ۑ
			writeFile(stamped, pdfFilePath);

			// ^CX^vς݃f[^̓Xg[
			stamped.close();

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	static void writeFile(InputStream in, String filePath) throws Exception {
		FileOutputStream out = new FileOutputStream(filePath + ".new.pdf");
		try {
			byte[] buf = new byte[4096];
			while (true) {
				int i = in.read(buf);
				if (i <= 0) {
					break;
				}
				out.write(buf, 0, i);
			}
		} finally {
			out.close();
		}
	}
}
