/*
 * SampleLTVPDFTimeStampGenerator.java
 *
 * Copyright(C) 2012-2020 AMANO Corporation. All Rights Reserved.
 *
 * PDF Lib-J API - TvvO
 *
 * w肳ꂽ PDF t@Cɑ΂ PDF ^CX^v𖄂ߍރR\[vOłB
 *
 * ZLeB̏̕ꍇ
 * Usage: java SampleLTVPDFTimeStampGenerator PDFFilePath LicenseFilePath Password CAFilePath CertificateFilePath
 *
 *    PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX
 *    LicenseFilePath   - CZXt@CpX
 *    Password          - CZXpX[h
 *    CAFilePath        - CAؖt@CpX
 *    CertificateFilePath  - [gؖt@CpX
 * 
 * PDF WZLeBL̏̕ꍇ
 * Usage: java SampleLTVPDFTimeStampGenerator PDFFilePath UserPassword OwnerPassword LicenseFilePath Password CAFilePath CertificateFilePath
 *
 *    PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX
 *    UserPassword      - [U[pX[h
 *    OwnerPassword     - I[i[pX[h
 *    LicenseFilePath   - CZXt@CpX
 *    Password          - CZXpX[h
 *    CAFilePath        - CAؖt@CpX
 *    CertificateFilePath  - [gؖt@CpX
 */

import java.io.*;
import java.util.*;
import java.security.cert.*;

import jp.co.amano.etiming.apl3161.*;
import jp.co.amano.etiming.atss3161.*;

public class SampleLTVPDFTimeStampGenerator {

	// A}m^CX^vǂ URL
	public static final String TIMESTAMP_SERVER = "http://tss3161.e-timing.ne.jp/astdtssvr/TSRequest";
	
	
	public static void main(String[] args) throws Exception {

		String pdfFilePath = null;
		char[] ownerPassword = null;
		char[] userPassword = null;
		String licenseFilePath = null;
		char[] password = null;

		String[] certFilePath = new String[2];
		certFilePath[0] = null;
		certFilePath[1] = null;
		
		String[] crlFilePath = new String[2];
		crlFilePath[0] = null;
		crlFilePath[1] = null;
		
		String HTTPURL = null;

		// `FbN
		if (args.length == 4 || args.length == 5) {
			pdfFilePath = args[0];
			licenseFilePath = args[1];
			password = args[2].toCharArray();
			certFilePath[0] = args[3];
			if (args.length == 5) {
				certFilePath[1] = args[4];
			}
		} else if (args.length == 6 || args.length == 7) {
			pdfFilePath = args[0];
			userPassword = args[1].toCharArray();
			ownerPassword = args[2].toCharArray();
			licenseFilePath = args[3];
			password = args[4].toCharArray();
			certFilePath[0] = args[5];
			if (args.length == 7) {
				certFilePath[1] = args[6];
			}
		} else if (args.length == 9 || args.length == 10) {
			pdfFilePath = args[0];
			userPassword = args[1].toCharArray();
			ownerPassword = args[2].toCharArray();
			licenseFilePath = args[3];
			password = args[4].toCharArray();
			certFilePath[0] = args[5];
			certFilePath[1] = args[6];
			crlFilePath[0] = args[7];
			crlFilePath[1] = args[8];
			if (args.length == 10) {
				HTTPURL = args[9];
			}
		} else {
			System.out.println("usage : java SampleLTVPDFTimeStampGenerator <PDFFilePath> <LicenseFilePath> <Password> <CertificateFilePath>");
			System.out.println("      : java SampleLTVPDFTimeStampGenerator <PDFFilePath> <LicenseFilePath> <Password> <CAFilePath> <CertificateFilePath>");
			System.out.println("      : java SampleLTVPDFTimeStampGenerator <PDFFilePath> <UserPassword> <OwnerPassword> <LicenseFilePath> <Password> <CertificateFilePath>");
			System.out.println("      : java SampleLTVPDFTimeStampGenerator <PDFFilePath> <UserPassword> <OwnerPassword> <LicenseFilePath> <Password> <CAFilePath> <CertificateFilePath>");
			System.out.println();
			System.out.println("      PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX");
			System.out.println("      UserPassword      - PDF ̃[U[pX[h");
			System.out.println("      OwnerPassword     - PDF ̃I[i[pX[h");
			System.out.println("      LicenseFilePath   - CZXt@CpX");
			System.out.println("      Password          - pX[h");
			System.out.println("      CAFilePath        - CAؖt@CpX");
			System.out.println("      CertificateFilePath  - [gؖt@CpX");
			System.exit(1);
		}

		try {
			// ^CX^vg[N擾ۂɎgp^CX^vvgR
			// IuWFNg𐶐
			LTVPDFTimeStampGenerator generator = null;
			if (HTTPURL != null) {
		        HTTPTimeStampProtocol protocol = new HTTPTimeStampProtocol("http://free-tsu.e-timing.ne.jp/TSS/HttpTspServer");

				//	PDF ^CX^vIuWFNg𐶐
				generator = new LTVPDFTimeStampGenerator(protocol);
			} else {
				AmanoTimeStampProtocol protocol =
					new AmanoTimeStampProtocol(TIMESTAMP_SERVER, readFile(licenseFilePath), password);
				// PDF ^CX^vIuWFNg𐶐
				generator = new LTVPDFTimeStampGenerator(protocol);
			}

			// ^CX^vg[N؃IuWFNg̐
			AmanoTSTValidator amanoValidator = null;
			if (crlFilePath[0] == null || crlFilePath[0].isEmpty()) {
				// pXgw肵Ȃꍇ
				amanoValidator = makeAmanoTSTValidator(certFilePath);
			} else {
				// pXgw肷ꍇ
				amanoValidator = makeAmanoTSTValidator(certFilePath, crlFilePath);
			}
			generator.setTSTValidator(amanoValidator);

			// PDF t@C̃I[v
			RandomAccessFile pdfDocument = new RandomAccessFile(pdfFilePath, "r");

			// PDF ^CX^v𐶐
			InputStream stamped = null;
			if (ownerPassword == null && userPassword == null) {
				stamped = generator.generate(pdfDocument);
			} else {
				stamped = generator.generate(pdfDocument, userPassword, ownerPassword);
			}
			
			int extensionResult = generator.getExtensionResult();
			
			System.out.println();
			if (extensionResult == 0) {
				System.out.println("Lԉ^CX^vƂɐALԉΏۂ̃^CX^v͑SāALԂ܂B");
			}
			else if (extensionResult == 2) {
				System.out.println("Lԉ^CX^vƂɐ܂Aꕔ̗LԉΏۂ̃^CX^v́ALԂ̉͂ł܂łB");
			}
			else {
				System.out.println("Lԉ^CX^vƂɐ܂ALԉΏۂ̃^CX^v͑SāALԂ̉͂ł܂łB");
			}
			pdfDocument.close();

			// PDF ^CX^v𖄂ߍ PDF t@C̕ۑ
			writeFile(stamped, pdfFilePath);

			// ^CX^vς݃f[^̓Xg[
			stamped.close();

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	// ^CX^vg[N؃IuWFNg𐶐
	//     CRL 擾ꍇɎgp
	private static AmanoTSTValidator makeAmanoTSTValidator(String[] certPath) throws Exception {

		// CRL 擾ɎgpvLV̐ݒ
		ProxyHost proxy = null;

		// CRL pP[^[쐬
		CRLDistributionPointsCRLLocator crlLocator = new CRLDistributionPointsCRLLocator();
		crlLocator.setConnectionTimeout(15000);
		crlLocator.setTimeout(15000);
		crlLocator.setProxyHost(proxy);
		crlLocator.setUpdateInterval(60);

		// ؖpP[^[쐬
		CollectionCertificateLocator certLocator = makeCollectionCertificateLocator(certPath);

		// ^CX^vg[N؂IuWFNg𐶐
		return new AmanoTSTValidator(certLocator, crlLocator);
	}


	// ^CX^vg[N؃IuWFNg𐶐
	//     CRL t@C擾ꍇɎgp
	private static AmanoTSTValidator makeAmanoTSTValidator(String[] certPath, String[] crlPath) throws Exception {

		// ؖt@Ng̍쐬
		CertificateFactory factory = CertificateFactory.getInstance("X.509");

		// CRL pP[^[쐬
		ArrayList crlList = new ArrayList();
		for (int i = 0; i < crlPath.length; i++) {
			if (crlPath[i] == null || crlPath[i].isEmpty()) {
				continue;
			}
			
			X509CRL crl = (X509CRL) factory.generateCRL(new FileInputStream(crlPath[i]));
			crlList.add((X509CRL)crl);
		}

		CollectionCRLLocator crlLocator = new CollectionCRLLocator(crlList);

		// ؖpP[^[IuWFNg𐶐
		CollectionCertificateLocator certLocator = makeCollectionCertificateLocator(certPath);

		// ^CX^vg[N؂IuWFNg𐶐
		return new AmanoTSTValidator(certLocator, crlLocator);
	}


	// ؖpP[^[IuWFNg𐶐
	private static CollectionCertificateLocator makeCollectionCertificateLocator(String[] certPath) throws Exception {

		// ؖt@Ng̍쐬
		CertificateFactory factory = CertificateFactory.getInstance("X.509");

		// ؖt@CǂݍŃXgɒǉ
		ArrayList certList = new ArrayList();
		for (int i = 0; i < certPath.length; i++) {
			if (certPath[i] == null || certPath[i].isEmpty()) {
				continue;
			}
			
			X509Certificate cert = (X509Certificate)factory.generateCertificate(new FileInputStream(certPath[i]));
			certList.add((X509Certificate)cert);
		}

		// ؖpP[^[IuWFNg𐶐
		return new CollectionCertificateLocator(certList);
	}

	static byte[] readFile(String filePath) throws Exception {
		InputStream in = new BufferedInputStream(new FileInputStream(filePath));
		try {
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			byte[] buf = new byte[4096];
			while (true) {
				int i = in.read(buf);
				if (i <= 0) {
					break;
				}
				out.write(buf, 0, i);
			}
			return out.toByteArray();
		} finally {
			in.close();
		}
	}

	static void writeFile(InputStream in, String filePath) throws Exception {
		FileOutputStream out = new FileOutputStream(filePath + ".new.pdf");
		try {
			byte[] buf = new byte[4096];
			while (true) {
				int i = in.read(buf);
				if (i <= 0) {
					break;
				}
				out.write(buf, 0, i);
			}
		} finally {
			out.close();
		}
	}
}
