/*
 * SamplePDFDocument.java
 *
 * Copyright(C) 2005-2022 AMANO Corporation. All Rights Reserved.
 *
 * PDF Lib-J API - TvvO
 *
 * w肳ꂽ PDF Ɋ܂܂^CX^v\
 * R\[vOłB
 *
 * ZLeB̏̕ꍇ
 * Usage: java SamplePDFDocument (TALL) PDFFilePath
 *
 *    PDFFilePath       - PDF t@CpX
 *
 * PDF WZLeBL̏̕ꍇ
 * Usage: java SamplePDFDocument (TALL) PDFFilePath UserPassword OwnerPassword
 *
 *    PDFFilePath       - PDF t@CpX
 *    UserPassword      - [U[pX[h
 *    OwnerPassword     - I[i[pX[h
 *
 * TALLw肵ꍇAA}m^CX^vT[rX 3161 ɂĔsꂽ^CX^vyуA}m^CX^vT[rX 3161 ȊO
 * sꂽ^CX^vg[N̂Ačؖ܂܂Ȃ PDF^CX^v\܂B
 */

import java.io.*;
import java.security.cert.*;
import java.text.MessageFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Date;
import java.util.ArrayList;

import jp.co.amano.etiming.apl3161.*;
import jp.co.amano.etiming.atss3161.*;

public class SamplePDFDocument {

	public static void main(String[] args) throws Exception {

		String pdfFilePath = null;
		char[] ownerPassword = null;
		char[] userPassword = null;
		
		int verifyTarget = PDFDocument.VERIFY_TARGET_AMANO;
		if (args.length == 1) {
			if (args[0].equals("TALL")) {
				showUsage();
			}
		}
		else if (args.length > 1) {
			if (args[0].equals("TALL")) {
				verifyTarget = PDFDocument.VERIFY_TARGET_FULL;
			}
		}

		if (verifyTarget == PDFDocument.VERIFY_TARGET_AMANO) {
			// `FbN
			if (args.length == 1) {
				pdfFilePath = args[0];
			} else if (args.length == 3) {
				pdfFilePath = args[0];
				userPassword = args[1].toCharArray();
				ownerPassword = args[2].toCharArray();
			} else {
				showUsage();
			}
		}
		else {
			// `FbN
			if (args.length == 2) {
				pdfFilePath = args[1];
			} else if (args.length == 4) {
				pdfFilePath = args[1];
				userPassword = args[2].toCharArray();
				ownerPassword = args[3].toCharArray();
			} else {
				showUsage();
			}
		}

		try {
			// PDF t@C̃I[v
			RandomAccessFile pdfDocument = new RandomAccessFile(pdfFilePath, "r");
			
			// ^CX^v̌ؑΏۂݒ肷
			PDFDocument.setVerifyTarget(verifyTarget);

			// PDF Ɋ܂܂ PDF ^CX^v擾IuWFNg𐶐
			PDFDocument pdfdoc = null;
			if (userPassword == null && ownerPassword == null ) {
				pdfdoc = new PDFDocument(pdfDocument);
			} else {
				pdfdoc = new PDFDocument(pdfDocument, userPassword, ownerPassword);
			}
			pdfDocument.close();

			PDFTimeStampWithVRI[] stampListWithVRI = pdfdoc.getTimeStampWithVRI();
			// PDF ^CX^vXg擾
			PDFTimeStamp[] stampList = pdfdoc.getTimeStamps();
			if (stampList.length == 0) {
				System.out.println("w肳ꂽ PDF ɂ̓^CX^v܂܂Ă܂B");
				System.exit(0);
			}

			// PDF ^CX^vXg̓e\
			for (int i = 0; i < stampList.length; i ++) {
				System.out.println("********** PDFTimeStamp[" + i +"] **********");
				

				// ^CX^vg[N̎擾
				TimeStampToken token  = stampList[i].getTimeStampToken();
				if (token == null) {
					System.out.println("Ή̃o[W̃^CX^v܂܂Ă܂B");
					System.out.println("");
					continue;
				}

				// ^CX^vg[Nf[^o͂
				outputTimeStampToken(token);

				// ^C][̎擾
				APLTimeZone timeZone = stampList[i].getTimeZone();

				// ^C][f[^̏o
				outputTimeZone(timeZone);

				// e̎擾
				PastedImprint pasted = stampList[i].getPastedImprint();
				if (pasted != null) {
					outputPastedImprint(pasted);
				}

				// DSS/VRI񂪑݂ꍇ́ATSA, CAACRL\܂B
				if (stampListWithVRI[i].getVRI() != null) {
					
					ArrayList certList = new ArrayList();
					ArrayList crlList = new ArrayList();
					certList.addAll(stampListWithVRI[i].getVRI().getCert());
					crlList.addAll(stampListWithVRI[i].getVRI().getCrl());

					for (int m = 0; m < certList.size(); m++) {
						X509Certificate cert = (X509Certificate)certList.get(m);
						
						boolean bTSA = false;
						if (m == certList.size() - 1) {
							bTSA = true;
						}
						outputCertInfo(cert, bTSA);
					}
					
					for (int j = 0; j < crlList.size(); j++) {
						X509CRL crl = (X509CRL)crlList.get(j);
						outputCrlInfo(crl);
					}
					
					if (certList != null) {
						certList.clear();
						certList = null;
					}
					if (crlList != null) {
						crlList.clear();
						crlList = null;
					}					
				}
				System.out.println("");
			}

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	// ^CX^vo͂
	private static void outputTimeStampToken(TimeStampToken token) throws Exception {

		TSTInfo tstInfo = token.getTSTInfo();
		System.out.println("TimeStampToken contents:");
		System.out.println("  TSTInfo:");
		System.out.println("    Version          - " + tstInfo.getVersion());
		System.out.println("    Policy           - " + tstInfo.getPolicy());
		System.out.println("    Serial Number    - " + tstInfo.getSerialNumber());
		System.out.println("    Nonce            - " + tstInfo.getNonce());
		MessageImprint mi = tstInfo.getMessageImprint();
		System.out.println("    Message Imprint:");
		System.out.println("      Algorithm      - " + mi.getHashAlgorithm().toString());
		System.out.println("      Hash Value     - " + toHexString(mi.getHashedMessage()));
		System.out.println("    Time:");
		System.out.println("      GenTime format - " + tstInfo.getGenTime().getTime());
		System.out.println("      Date format    - " + tstInfo.getGenTime().getDate());
	}

	// ^C][̏o͂s
	private static void outputTimeZone(APLTimeZone timeZone) {
		System.out.println();
		System.out.println("TimeZone contents:");
		System.out.println("  Name               - " + timeZone.getName());
		System.out.println("  Offset             - " + timeZone.getOffset());
	}

	// ȅo͂s
	private static void outputPastedImprint(PastedImprint pasted) {
		System.out.println();
		System.out.println("PastedImprint contents:");
		System.out.println("  Page               - " + pasted.getPage());

		APLPosition position = pasted.getPosition();
		System.out.println("  APLPosition:");
		System.out.println("    Origin           - " + position.getOrigin());
		System.out.println("    Unit             - " + position.getUnit());
		System.out.println("    X                - " + position.getX());
		System.out.println("    Y                - " + position.getY());

		Imprint imprint = pasted.getImprint();
		TimeFaceFormat format = imprint.getTimeFaceFormat();
		System.out.println("  Imprint:");
		System.out.println("    Shape            - " + imprint.getShape());
		System.out.println("    Background       - " + imprint.getBackground());
		System.out.println("    LinkableAttached - " + imprint.isLinkableAttached());
		System.out.println("    TimeFaceFormat:");
		System.out.println("      Date           - " + format.getDate());
		System.out.println("      Time           - " + format.getTime());

	}
	
	// TSA, CȀo͂s
	private static void outputCertInfo(X509Certificate cert, boolean bTsa) {
		try {
		System.out.println();
		if (bTsa == true) {
			System.out.println("TSA contents:");
		}
		else {
			System.out.println("CA contents:");
		}
		
		System.out.println("  Serial Number                   - " + cert.getSerialNumber());
		System.out.println("  Not Before                      - " + getTime(cert.getNotBefore()));
		System.out.println("  Not After                       - " + getTime(cert.getNotAfter()));
		// s
		System.out.println("  Holder                          - " + cert.getSubjectDN().getName());
		// s
		System.out.println("  Issuer                          - " + cert.getIssuerDN().getName());
		// l
		String outString = toHexString(cert.getSignature()).toUpperCase();
		System.out.println("  Signature Value                 - " + outString);

		// 
		System.out.println("  Signature Length                - " + (cert.getSignature().length * 8));
		// ASYOID
		System.out.println("  Signature Algorithm Identifier: - " + cert.getSigAlgOID());
		// ASY
		System.out.println("  Signature Algorithm             - " + cert.getSigAlgName());

		} catch (Exception ex) {
			ex.printStackTrace();
		}		
	}
	
	// CRL̏o͂s
	private static void outputCrlInfo(X509CRL crl) {
		try {
		System.out.println();
		System.out.println("CRL contents:");
		System.out.println("  Issuer                  - " + crl.getIssuerDN().toString());
		System.out.println("  This Update             - " + getTime(crl.getThisUpdate()));
		System.out.println("  Next Update             - " + getTime(crl.getNextUpdate()));
		System.out.println("  Signature Algorithm     - " + crl.getSigAlgName());	

		} catch (Exception ex) {
			ex.printStackTrace();
		}		
	}

	private static String toHexString(byte[] bytes) {
		String str = "";
		try {
			DataInputStream in = new DataInputStream(new ByteArrayInputStream(bytes));
			int i;
			while (in.available() > 0) {
				i = in.readUnsignedByte();
				if (i < 0x10) {
					str += "0";
				}
				str += Integer.toHexString(i);
			}
		} catch (IOException e) {
		}
		return str;
	}
	
	private static String getTime(Date date) {
		Calendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		Object dateval[] = { new Integer(calendar.get(Calendar.YEAR)),
				new Integer(calendar.get(Calendar.MONTH)),
				new Integer(calendar.get(Calendar.DATE)), };

		Object timeval[] = { new Integer(calendar.get(Calendar.HOUR_OF_DAY)),
				new Integer(calendar.get(Calendar.MINUTE)),
				new Integer(calendar.get(Calendar.SECOND)) };

		String strDate = MessageFormat.format(
				"{0,number,0000}{1,number,00}{2,number,00}", dateval);
		String strTime = MessageFormat.format(
				"{0,number,00}{1,number,00}{2,number,00}", timeval);

		StringBuffer buf = new StringBuffer();
		buf.append(strDate);
		buf.append(strTime);
		buf.append("Z");
		return buf.toString();
	}
	
	private static void showUsage() {
		System.out.println("usage : java SamplePDFDocument (TALL) <PDFFilePath>");
		System.out.println("      : java SamplePDFDocument (TALL) <PDFFilePath> <UserPassword> <OwnerPassword>");
		System.out.println();
		System.out.println("      PDFFilePath      - PDF t@CpX");
		System.out.println("      UserPassword     - [U[pX[h");
		System.out.println("      OwnerPassword    - I[i[pX[h");
		System.exit(1);
	}
}
