/*
 * SamplePDFTimeStampGenerator.java
 *
 * Copyright(C) 2005-2006 AMANO Corporation. All Rights Reserved.
 *
 * PDF Lib-J API - TvvO
 *
 * w肳ꂽ PDF t@Cɑ΂ PDF ^CX^v𖄂ߍރR\[vOłB
 *
 * ZLeB̏̕ꍇ
 * Usage: java SamplePDFTimeStampGenerator PDFPDFFilePath LicenseFilePath Password
 *
 *    PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX
 *    LicenseFilePath   - CZXt@CpX
 *    Password          - CZXpX[h
 *
 * PDF WZLeBL̏̕ꍇ
 * Usage: java SamplePDFTimeStampGenerator PDFFilePath UserPassword OwnerPassword LicenseFilePath Password
 *
 *    PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX
 *    UserPassword      - [U[pX[h
 *    OwnerPassword     - I[i[pX[h
 *    LicenseFilePath   - CZXt@CpX
 *    Password          - CZXpX[h
 */

import java.io.*;
import java.util.*;
import java.security.cert.*;
import jp.co.amano.etiming.apl3161.*;

public class SamplePDFTimeStampGenerator {

	// A}m^CX^vǂ URL
    public static final String TIMESTAMP_SERVER = "http://tss3161.e-timing.ne.jp/astdtssvr/TSRequest";

	public static void main(String[] args) throws Exception {

		String pdfFilePath = null;
		char[] ownerPassword = null;
		char[] userPassword = null;
		String licenseFilePath = null;
		char[] password = null;

		// `FbN
		if (args.length == 3) {
			pdfFilePath = args[0];
			licenseFilePath = args[1];
			password = args[2].toCharArray();
		} else if (args.length == 5) {
			pdfFilePath = args[0];
			userPassword = args[1].toCharArray();
			ownerPassword = args[2].toCharArray();
			licenseFilePath = args[3];
			password = args[4].toCharArray();
		} else {
			System.out.println("usage : java SamplePDFTimeStampGenerator <PDFFilePath> <LicenseFilePath> <Password>");
			System.out.println("      : java SamplePDFTimeStampGenerator <PDFFilePath> <UserPassword> <OwnerPassword> <LicenseFilePath> <Password>");
			System.out.println();
			System.out.println("      PDFFilePath       - PDF ^CX^v𖄂ߍ PDF t@CpX");
			System.out.println("      UserPassword      - PDF ̃[U[pX[h");
			System.out.println("      OwnerPassword     - PDF ̃I[i[pX[h");
			System.out.println("      LicenseFilePath   - CZXt@CpX");
			System.out.println("      Password          - pX[h");
			System.exit(1);
		}

		try {
			// ^CX^vg[N擾ۂɎgp^CX^vvgR
			// IuWFNg𐶐
			AmanoTimeStampProtocol protocol =
				new AmanoTimeStampProtocol(TIMESTAMP_SERVER, readFile(licenseFilePath), password);

			// PDF ^CX^vIuWFNg𐶐
			PDFTimeStampGenerator generator = new PDFTimeStampGenerator(protocol);

			// ^C][ݒ
			generator.setTimeZone(DefinedTimeZone.JST);

			// ^CX^vg[N؃IuWFNg̐
			NoTSACertValidationTSTValidator validator = new NoTSACertValidationTSTValidator();
			generator.setTSTValidator(validator);

			// eȂ
			generator.setImprintPasting(null);

			// e
			// ê PDF ^CX^v𐶐ɂ͎̃RgAEgO
			//ImprintPastingParameters params = makeImprintPastingParameters();
			//generator.setImprintPasting(params);

			// PDF t@C̃I[v
			RandomAccessFile pdfDocument = new RandomAccessFile(pdfFilePath, "r");

			// PDF ^CX^v𐶐
			InputStream stamped = null;
			if (ownerPassword == null && userPassword == null) {
				stamped = generator.generate(pdfDocument);
			} else {
				stamped = generator.generate(pdfDocument, userPassword, ownerPassword);
			}
			pdfDocument.close();

			// PDF ^CX^v𖄂ߍ PDF t@C̕ۑ
			writeFile(stamped, pdfFilePath);

			// ^CX^vς݃f[^̓Xg[
			stamped.close();

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}


	/**
	 * ftHgݒ PDFTimeStampGenerator ̃CX^X쐬
	 */
	static PDFTimeStampGenerator makeAmanoInstance(String licenseFilePath,
												   char[] password,
												   String certPath)
	throws Exception {

		// ؖt@Ng̍쐬
		CertificateFactory factory = CertificateFactory.getInstance("X.509");

		// ؖt@CǂݍŃXgɒǉ
		ArrayList certList = new ArrayList();
		X509Certificate cert = (X509Certificate)factory.generateCertificate(new FileInputStream(certPath));
		certList.add((X509Certificate)cert);

		// PDF ^CX^vIuWFNg𐶐
		return PDFTimeStampGenerator.getAmanoInstance(readFile(licenseFilePath),
													  password,
													  DefinedTimeZone.JST,
													  certList,
													  null);
	}


	/**
	 * ȅݒ肷
	 */
	static ImprintPastingParameters makeImprintPastingParameters() {
		ImprintPastingParameters params = new ImprintPastingParameters();

		// o̓y[W̐ݒ
		//   y[W 0 JEg
		params.setPage(0);

		// y[W𐔂Zbg
		//      ImprintPastingParameters.FIRST_PAGE   ŏ̃y[W
		//      ImprintPastingParameters.LAST_PAGE    Ō̃y[W
		params.setPageOrigin(ImprintPastingParameters.FIRST_PAGE);

		// e̎\@̐ݒ
		//   t̕\`  1  15 ܂ł̐Ŏw
		//   ̕\`  1  4 ܂ł̐Ŏw
		TimeFaceFormat timeFormat = new TimeFaceFormat(1, 1);

		// e̐ݒ
		// shape
		//		Imprint.CIRCULAR     ی^
		//		Imprint.RECTANGULAR  p`
		// background
		//		Imprint.OPAQUE                  s
		//		Imprint.TRANSPARENT             
		//		Imprint.TRANSLUCENT_FOREGROUND  Oi
		// linkableAttached
		//      true    JuL
		//      false   Ju
		Imprint imprint = new Imprint(Imprint.CIRCULAR, Imprint.OPAQUE, timeFormat, true);
		params.setImprint(imprint);

		// \W̐ݒ
		// origin
		//     APLPosition.UPPER_LEFT_ORIGIN  _
		//     APLPosition.LOWER_LEFT_ORIGIN  _
		// unit
		//     APLPosition.MM     ~[gP
		//     APLPosition.INCH   C`P
		//     APLPosition.POINT  |CgP
		APLPosition position = new APLPosition(APLPosition.UPPER_LEFT_ORIGIN,
											   APLPosition.MM,
											   2.0,
											   2.0);
		params.setPosition(position);

		return params;
	}

	static byte[] readFile(String filePath) throws Exception {
		InputStream in = new BufferedInputStream(new FileInputStream(filePath));
		try {
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			byte[] buf = new byte[4096];
			while (true) {
				int i = in.read(buf);
				if (i <= 0) {
					break;
				}
				out.write(buf, 0, i);
			}
			return out.toByteArray();
		} finally {
			in.close();
		}
	}

	static void writeFile(InputStream in, String filePath) throws Exception {
		FileOutputStream out = new FileOutputStream(filePath + ".new.pdf");
		try {
			byte[] buf = new byte[4096];
			while (true) {
				int i = in.read(buf);
				if (i <= 0) {
					break;
				}
				out.write(buf, 0, i);
			}
		} finally {
			out.close();
		}
	}
}
